@@ -1,28 +1,58 @@
 import contextlib
 import hashlib
+import json
 import os
 import subprocess
 import sys
+import tempfile
 import warnings
 from functools import lru_cache
-
-import crayons
-from click import echo as click_echo
+from pathlib import Path
+from typing import Dict, List, Optional, Set, Tuple, Union
 
 from pipenv import environments
+from pipenv._compat import decode_for_output
 from pipenv.exceptions import RequirementError, ResolutionFailure
-from pipenv.vendor.requirementslib import Pipfile, Requirement
+from pipenv.patched.pip._internal.cache import WheelCache
+from pipenv.patched.pip._internal.commands.install import InstallCommand
+from pipenv.patched.pip._internal.exceptions import InstallationError
+from pipenv.patched.pip._internal.models.target_python import TargetPython
+from pipenv.patched.pip._internal.network.cache import SafeFileCache
+from pipenv.patched.pip._internal.operations.build.build_tracker import (
+    get_build_tracker,
+)
+from pipenv.patched.pip._internal.req.constructors import (
+    install_req_from_parsed_requirement,
+)
+from pipenv.patched.pip._internal.req.req_file import parse_requirements
+from pipenv.patched.pip._internal.utils.hashes import FAVORITE_HASH
+from pipenv.patched.pip._internal.utils.temp_dir import global_tempdir_manager
+from pipenv.patched.pip._vendor import pkg_resources
+from pipenv.project import Project
+from pipenv.vendor import click
+from pipenv.vendor.requirementslib import Requirement
+from pipenv.vendor.requirementslib.models.requirements import Line
 from pipenv.vendor.requirementslib.models.utils import DIRECT_URL_RE
 from pipenv.vendor.vistir import TemporaryDirectory, open_file
 from pipenv.vendor.vistir.path import create_tracked_tempdir
 
+try:
+    # this is only in Python3.8 and later
+    from functools import cached_property
+except ImportError:
+    # eventually distlib will remove cached property when they drop Python3.7
+    from pipenv.patched.pip._vendor.distlib.util import cached_property
+
 from .dependencies import (
     HackedPythonVersion,
     clean_pkg_version,
     convert_deps_to_pip,
+    get_constraints_from_deps,
+    get_lockfile_section_using_pipfile_category,
     get_vcs_deps,
     is_pinned_requirement,
     pep423_name,
+    prepare_constraint_file,
     translate_markers,
 )
 from .indexes import parse_indexes, prepare_pip_source_args
@@ -31,12 +61,34 @@ from .locking import format_requirement_for_lockfile, prepare_lockfile
 from .shell import make_posix, subprocess_run, temp_environ
 from .spinner import create_spinner
 
-if environments.MYPY_RUNNING:
-    from typing import Any, Dict, List, Optional, Set, Tuple, Union  # noqa
 
-    from pipenv.project import Project  # noqa
-    from pipenv.vendor.requirementslib import Pipfile, Requirement  # noqa
-    from pipenv.vendor.requirementslib.models.requirements import Line  # noqa
+def get_package_finder(
+    install_cmd=None,
+    options=None,
+    session=None,
+    platform=None,
+    python_versions=None,
+    abi=None,
+    implementation=None,
+    ignore_requires_python=None,
+):
+    """Reduced Shim for compatibility to generate package finders."""
+    py_version_info = None
+    if python_versions:
+        py_version_info_python = max(python_versions)
+        py_version_info = tuple([int(part) for part in py_version_info_python])
+    target_python = TargetPython(
+        platforms=[platform] if platform else None,
+        py_version_info=py_version_info,
+        abis=[abi] if abi else None,
+        implementation=implementation,
+    )
+    return install_cmd._build_package_finder(
+        options=options,
+        session=session,
+        target_python=target_python,
+        ignore_requires_python=ignore_requires_python,
+    )
 
 
 class HashCacheMixin:
@@ -64,9 +116,7 @@ class HashCacheMixin:
         return hash_value.decode("utf8")
 
     def _get_file_hash(self, link):
-        from pipenv.vendor.pip_shims import shims
-
-        h = hashlib.new(shims.FAVORITE_HASH)
+        h = hashlib.new(FAVORITE_HASH)
         with open_file(link.url, self.session) as fp:
             for chunk in iter(lambda: fp.read(8096), b""):
                 h.update(chunk)
@@ -85,6 +135,7 @@ class Resolver:
         skipped=None,
         clear=False,
         pre=False,
+        category=None,
     ):
         self.initial_constraints = constraints
         self.req_dir = req_dir
@@ -94,6 +145,7 @@ class Resolver:
         self.hashes = {}
         self.clear = clear
         self.pre = pre
+        self.category = category
         self.results = None
         self.markers_lookup = markers_lookup if markers_lookup is not None else {}
         self.index_lookup = index_lookup if index_lookup is not None else {}
@@ -122,55 +174,45 @@ class Resolver:
     @staticmethod
     @lru_cache()
     def _get_pip_command():
-        from pipenv.vendor.pip_shims import shims
-
-        return shims.InstallCommand()
+        return InstallCommand(name="InstallCommand", summary="pip Install command.")
 
     @property
     def hash_cache(self):
-        from pipenv.vendor.pip_shims import shims
-
         if not self._hash_cache:
-            self._hash_cache = type(
-                "HashCache", (HashCacheMixin, shims.SafeFileCache), {}
-            )(os.path.join(self.project.s.PIPENV_CACHE_DIR, "hashes"), self.session)
+            self._hash_cache = type("HashCache", (HashCacheMixin, SafeFileCache), {})(
+                os.path.join(self.project.s.PIPENV_CACHE_DIR, "hashes"), self.session
+            )
         return self._hash_cache
 
-    @classmethod
     def get_metadata(
-        cls,
-        deps,  # type: List[str]
-        index_lookup,  # type: Dict[str, str]
-        markers_lookup,  # type: Dict[str, str]
-        project,  # type: Project
-        sources,  # type: Dict[str, str]
-        req_dir=None,  # type: Optional[str]
-        pre=False,  # type: bool
-        clear=False,  # type: bool
-    ):
-        # type: (...) -> Tuple[Set[str], Dict[str, Dict[str, Union[str, bool, List[str]]]], Dict[str, str], Dict[str, str]]
-        constraints = set()  # type: Set[str]
-        skipped = {}  # type: Dict[str, Dict[str, Union[str, bool, List[str]]]]
+        self,
+        deps: List[str],
+        index_lookup: Dict[str, str],
+        markers_lookup: Dict[str, str],
+        project: Project,
+        sources: Dict[str, str],
+        req_dir: Optional[str] = None,
+        pre: bool = False,
+        clear: bool = False,
+        category: str = None,
+    ) -> Tuple[
+        Set[str],
+        Dict[str, Dict[str, Union[str, bool, List[str]]]],
+        Dict[str, str],
+        Dict[str, str],
+    ]:
+        constraints: Set[str] = set()
+        skipped: Dict[str, Dict[str, Union[str, bool, List[str]]]] = {}
         if index_lookup is None:
             index_lookup = {}
         if markers_lookup is None:
             markers_lookup = {}
         if not req_dir:
             req_dir = create_tracked_tempdir(prefix="pipenv-", suffix="-reqdir")
-        transient_resolver = cls(
-            [],
-            req_dir,
-            project,
-            sources,
-            index_lookup=index_lookup,
-            markers_lookup=markers_lookup,
-            clear=clear,
-            pre=pre,
-        )
         for dep in deps:
             if not dep:
                 continue
-            req, req_idx, markers_idx = cls.parse_line(
+            req, req_idx, markers_idx = self.parse_line(
                 dep,
                 index_lookup=index_lookup,
                 markers_lookup=markers_lookup,
@@ -189,7 +231,7 @@ class Resolver:
                 )
             if not use_sources:
                 use_sources = sources
-            transient_resolver = cls(
+            transient_resolver = Resolver(
                 [],
                 req_dir,
                 project,
@@ -198,23 +240,22 @@ class Resolver:
                 markers_lookup=markers_lookup,
                 clear=clear,
                 pre=pre,
+                category=category,
             )
-            constraint_update, lockfile_update = cls.get_deps_from_req(
+            constraint_update, lockfile_update = self.get_deps_from_req(
                 req, resolver=transient_resolver, resolve_vcs=project.s.PIPENV_RESOLVE_VCS
             )
             constraints |= constraint_update
             skipped.update(lockfile_update)
         return constraints, skipped, index_lookup, markers_lookup
 
-    @classmethod
     def parse_line(
-        cls,
-        line,  # type: str
-        index_lookup=None,  # type: Dict[str, str]
-        markers_lookup=None,  # type: Dict[str, str]
-        project=None,  # type: Optional[Project]
-    ):
-        # type: (...) -> Tuple[Requirement, Dict[str, str], Dict[str, str]]
+        self,
+        line: str,
+        index_lookup: Dict[str, str] = None,
+        markers_lookup: Dict[str, str] = None,
+        project: Optional[Project] = None,
+    ) -> Tuple[Requirement, Dict[str, str], Dict[str, str]]:
 
         if index_lookup is None:
             index_lookup = {}
@@ -226,7 +267,7 @@ class Resolver:
             project = Project()
         index, extra_index, trust_host, remainder = parse_indexes(line)
         line = " ".join(remainder)
-        req = None  # type: Requirement
+        req: Requirement = None
         try:
             req = Requirement.from_line(line)
         except ValueError:
@@ -261,9 +302,12 @@ class Resolver:
             markers_lookup[req.normalized_name] = req.markers.replace('"', "'")
         return req, index_lookup, markers_lookup
 
-    @classmethod
-    def get_deps_from_req(cls, req, resolver=None, resolve_vcs=True):
-        # type: (Requirement, Optional["Resolver"], bool) -> Tuple[Set[str], Dict[str, Dict[str, Union[str, bool, List[str]]]]]
+    def get_deps_from_req(
+        self,
+        req: Requirement,
+        resolver: Optional["Resolver"] = None,
+        resolve_vcs: bool = True,
+    ) -> Tuple[Set[str], Dict[str, Dict[str, Union[str, bool, List[str]]]]]:
         from pipenv.vendor.requirementslib.models.requirements import Requirement
         from pipenv.vendor.requirementslib.models.utils import (
             _requirement_to_str_lowercase_name,
@@ -271,8 +315,9 @@ class Resolver:
         from pipenv.vendor.requirementslib.utils import is_installable_dir
 
         # TODO: this is way too complex, refactor this
-        constraints = set()  # type: Set[str]
-        locked_deps = {}  # type: Dict[str, Dict[str, Union[str, bool, List[str]]]]
+        constraints: Set[str] = set()
+        locked_deps: Dict[str, Dict[str, Union[str, bool, List[str]]]] = {}
+        editable_packages = self.project.get_editable_packages(category=self.category)
         if (req.is_file_or_url or req.is_vcs) and not req.is_wheel:
             # for local packages with setup.py files and potential direct url deps:
             if req.is_vcs:
@@ -281,7 +326,7 @@ class Resolver:
                 entry = lockfile[pep423_name(req.normalized_name)]
             else:
                 _, entry = req.pipfile_entry
-            parsed_line = req.req.parsed_line  # type: Line
+            parsed_line: Line = req.req.parsed_line
             try:
                 name = req.normalized_name
             except TypeError:
@@ -308,21 +353,22 @@ class Resolver:
                         if not r.url:
                             continue
                         line = _requirement_to_str_lowercase_name(r)
-                        new_req, _, _ = cls.parse_line(line)
+                        new_req, _, _ = self.parse_line(line)
                         if r.marker and not r.marker.evaluate():
                             new_constraints = {}
                             _, new_entry = req.pipfile_entry
                             new_lock = {pep423_name(new_req.normalized_name): new_entry}
                         else:
-                            new_constraints, new_lock = cls.get_deps_from_req(
+                            new_constraints, new_lock = self.get_deps_from_req(
                                 new_req, resolver
                             )
                         locked_deps.update(new_lock)
                         constraints |= new_constraints
                 # if there is no marker or there is a valid marker, add the constraint line
                 elif r and (not r.marker or (r.marker and r.marker.evaluate())):
-                    line = _requirement_to_str_lowercase_name(r)
-                    constraints.add(line)
+                    if r.name not in editable_packages:
+                        line = _requirement_to_str_lowercase_name(r)
+                        constraints.add(line)
             # ensure the top level entry remains as provided
             # note that we shouldn't pin versions for editable vcs deps
             if not req.is_vcs:
@@ -356,15 +402,16 @@ class Resolver:
                     hashes = resolver.collect_hashes(ireq) if resolver else []
                     new_req = Requirement.from_ireq(ireq)
                     new_req = new_req.add_hashes(hashes)
+                    new_req = new_req.merge_markers(req.markers)
                     name, entry = new_req.pipfile_entry
                     locked_deps[pep423_name(name)] = translate_markers(entry)
-                    click_echo(
+                    click.echo(
                         "{} doesn't match your environment, "
                         "its dependencies won't be resolved.".format(req.as_line()),
                         err=True,
                     )
                 else:
-                    click_echo(
+                    click.echo(
                         "Could not find a version of {} that matches your environment, "
                         "it will be skipped.".format(req.as_line()),
                         err=True,
@@ -377,16 +424,16 @@ class Resolver:
     @classmethod
     def create(
         cls,
-        deps,  # type: List[str]
-        project,  # type: Project
-        index_lookup=None,  # type: Dict[str, str]
-        markers_lookup=None,  # type: Dict[str, str]
-        sources=None,  # type: List[str]
-        req_dir=None,  # type: str
-        clear=False,  # type: bool
-        pre=False,  # type: bool
-    ):
-        # type: (...) -> "Resolver"
+        deps: List[str],
+        project: Project,
+        index_lookup: Dict[str, str] = None,
+        markers_lookup: Dict[str, str] = None,
+        sources: List[str] = None,
+        req_dir: str = None,
+        clear: bool = False,
+        pre: bool = False,
+        category: str = None,
+    ) -> "Resolver":
 
         if not req_dir:
             req_dir = create_tracked_tempdir(suffix="-requirements", prefix="pipenv-")
@@ -396,7 +443,18 @@ class Resolver:
             markers_lookup = {}
         if sources is None:
             sources = project.sources
-        constraints, skipped, index_lookup, markers_lookup = cls.get_metadata(
+        resolver = Resolver(
+            [],
+            req_dir,
+            project,
+            sources,
+            index_lookup=index_lookup,
+            markers_lookup=markers_lookup,
+            clear=clear,
+            pre=pre,
+            category=category,
+        )
+        constraints, skipped, index_lookup, markers_lookup = resolver.get_metadata(
             deps,
             index_lookup,
             markers_lookup,
@@ -405,52 +463,13 @@ class Resolver:
             req_dir=req_dir,
             pre=pre,
             clear=clear,
-        )
-        return Resolver(
-            constraints,
-            req_dir,
-            project,
-            sources,
-            index_lookup=index_lookup,
-            markers_lookup=markers_lookup,
-            skipped=skipped,
-            clear=clear,
-            pre=pre,
-        )
-
-    @classmethod
-    def from_pipfile(cls, project, pipfile=None, dev=False, pre=False, clear=False):
-        # type: (Optional[Project], Optional[Pipfile], bool, bool, bool) -> "Resolver"
-
-        if not pipfile:
-            pipfile = project._pipfile
-        req_dir = create_tracked_tempdir(suffix="-requirements", prefix="pipenv-")
-        index_lookup, markers_lookup = {}, {}
-        deps = set()
-        if dev:
-            deps.update({req.as_line() for req in pipfile.dev_packages})
-        deps.update({req.as_line() for req in pipfile.packages})
-        constraints, skipped, index_lookup, markers_lookup = cls.get_metadata(
-            list(deps),
-            index_lookup,
-            markers_lookup,
-            project,
-            project.sources,
-            req_dir=req_dir,
-            pre=pre,
-            clear=clear,
-        )
-        return Resolver(
-            constraints,
-            req_dir,
-            project,
-            project.sources,
-            index_lookup=index_lookup,
-            markers_lookup=markers_lookup,
-            skipped=skipped,
-            clear=clear,
-            pre=pre,
-        )
+            category=category,
+        )  # Workaround to the fact `get_metadata` instantiates a transient Resolver
+        resolver.initial_constraints = constraints
+        resolver.skipped = skipped
+        resolver.index_lookup = index_lookup
+        resolver.markers_lookup = markers_lookup
+        return resolver
 
     @property
     def pip_command(self):
@@ -482,29 +501,13 @@ class Resolver:
         return self._pip_args
 
     def prepare_constraint_file(self):
-        from pipenv.vendor.vistir.path import create_tracked_tempfile
-
-        constraints_file = create_tracked_tempfile(
-            mode="w",
-            prefix="pipenv-",
-            suffix="-constraints.txt",
-            dir=self.req_dir,
-            delete=False,
+        constraint_filename = prepare_constraint_file(
+            self.initial_constraints,
+            directory=self.req_dir,
+            sources=self.sources,
+            pip_args=self.pip_args,
         )
-        skip_args = ("build-isolation", "use-pep517", "cache-dir")
-        args_to_add = [
-            arg
-            for arg in self.pip_args
-            if not any(bad_arg in arg for bad_arg in skip_args)
-        ]
-        if self.sources:
-            requirementstxt_sources = " ".join(args_to_add) if args_to_add else ""
-            requirementstxt_sources = requirementstxt_sources.replace(" --", "\n--")
-            constraints_file.write(f"{requirementstxt_sources}\n")
-        constraints = self.initial_constraints
-        constraints_file.write("\n".join([c for c in constraints]))
-        constraints_file.close()
-        return constraints_file.name
+        return constraint_filename
 
     @property
     def constraint_file(self):
@@ -512,6 +515,17 @@ class Resolver:
             self._constraint_file = self.prepare_constraint_file()
         return self._constraint_file
 
+    @cached_property
+    def default_constraint_file(self):
+        default_constraints = get_constraints_from_deps(self.project.packages)
+        default_constraint_filename = prepare_constraint_file(
+            default_constraints,
+            directory=self.req_dir,
+            sources=None,
+            pip_args=None,
+        )
+        return default_constraint_filename
+
     @property
     def pip_options(self):
         if self._pip_options is None:
@@ -546,10 +560,8 @@ class Resolver:
 
     @property
     def finder(self):
-        from pipenv.vendor.pip_shims import shims
-
         if self._finder is None:
-            self._finder = shims.get_package_finder(
+            self._finder = get_package_finder(
                 install_cmd=self.pip_command,
                 options=self.pip_options,
                 session=self.session,
@@ -561,10 +573,8 @@ class Resolver:
 
     @property
     def ignore_compatibility_finder(self):
-        from pipenv.vendor.pip_shims import shims
-
         if self._ignore_compatibility_finder is None:
-            ignore_compatibility_finder = shims.get_package_finder(
+            ignore_compatibility_finder = get_package_finder(
                 install_cmd=self.pip_command,
                 options=self.pip_options,
                 session=self.session,
@@ -583,12 +593,10 @@ class Resolver:
 
     @property
     def parsed_constraints(self):
-        from pipenv.vendor.pip_shims import shims
-
         pip_options = self.pip_options
         pip_options.extra_index_urls = []
         if self._parsed_constraints is None:
-            self._parsed_constraints = shims.parse_requirements(
+            self._parsed_constraints = parse_requirements(
                 self.constraint_file,
                 finder=self.finder,
                 session=self.session,
@@ -596,12 +604,33 @@ class Resolver:
             )
         return self._parsed_constraints
 
+    @cached_property
+    def parsed_default_constraints(self):
+        pip_options = self.pip_options
+        pip_options.extra_index_urls = []
+        parsed_default_constraints = parse_requirements(
+            self.default_constraint_file,
+            constraint=True,
+            finder=self.finder,
+            session=self.session,
+            options=pip_options,
+        )
+        return parsed_default_constraints
+
+    @cached_property
+    def default_constraints(self):
+        default_constraints = [
+            install_req_from_parsed_requirement(
+                c,
+                isolated=self.pip_options.build_isolation,
+                user_supplied=False,
+            )
+            for c in self.parsed_default_constraints
+        ]
+        return default_constraints
+
     @property
     def constraints(self):
-        from pipenv.patched.notpip._internal.req.constructors import (
-            install_req_from_parsed_requirement,
-        )
-
         if self._constraints is None:
             self._constraints = [
                 install_req_from_parsed_requirement(
@@ -612,17 +641,15 @@ class Resolver:
                 )
                 for c in self.parsed_constraints
             ]
+            # Only use default_constraints when installing dev-packages
+            if self.category != "packages":
+                self._constraints += self.default_constraints
+            self._constraints.sort(key=lambda ireq: ireq.name)
         return self._constraints
 
     @contextlib.contextmanager
     def get_resolver(self, clear=False):
-        from pipenv.vendor.pip_shims.shims import (
-            WheelCache,
-            get_requirement_tracker,
-            global_tempdir_manager,
-        )
-
-        with global_tempdir_manager(), get_requirement_tracker() as req_tracker, TemporaryDirectory(
+        with global_tempdir_manager(), get_build_tracker() as build_tracker, TemporaryDirectory(
             suffix="-build", prefix="pipenv-"
         ) as directory:
             pip_options = self.pip_options
@@ -632,7 +659,7 @@ class Resolver:
             preparer = self.pip_command.make_requirement_preparer(
                 temp_build_dir=directory,
                 options=pip_options,
-                req_tracker=req_tracker,
+                build_tracker=build_tracker,
                 session=self.session,
                 finder=finder,
                 use_user_site=False,
@@ -652,9 +679,6 @@ class Resolver:
             yield resolver
 
     def resolve(self):
-        from pipenv.exceptions import ResolutionFailure
-        from pipenv.vendor.pip_shims.shims import InstallationError
-
         self.constraints  # For some reason it is important to evaluate constraints before resolver context
         with temp_environ(), self.get_resolver() as resolver:
             try:
@@ -680,11 +704,20 @@ class Resolver:
                 if candidate:
                     requires_python = candidate.link.requires_python
                     if requires_python:
-                        marker = marker_from_specifier(requires_python)
-                        self.markers[result.name] = marker
-                        result.markers = marker
-                        if result.req:
-                            result.req.marker = marker
+                        try:
+
+                            marker = marker_from_specifier(requires_python)
+                            self.markers[result.name] = marker
+                            result.markers = marker
+                            if result.req:
+                                result.req.marker = marker
+                        except TypeError as e:
+                            click.echo(
+                                f"Error generating python marker for {candidate}.  "
+                                f"Is the specifier {requires_python} incorrectly quoted or otherwise wrong?"
+                                f"Full error: {e}",
+                                err=True,
+                            )
             new_tree.add(result)
         self.resolved_tree = new_tree
 
@@ -700,8 +733,6 @@ class Resolver:
         return cleaned_checksums
 
     def _get_hashes_from_pypi(self, ireq):
-        from pipenv.vendor.pip_shims import shims
-
         pkg_url = f"https://pypi.org/pypi/{ireq.name}/json"
         session = _get_requests_session(self.project.s.PIPENV_MAX_RETRIES)
         try:
@@ -719,13 +750,13 @@ class Resolver:
                 if spec:
                     version = spec.version
             for release in cleaned_releases[version]:
-                collected_hashes.add(release["digests"][shims.FAVORITE_HASH])
-            return self.prepend_hash_types(collected_hashes, shims.FAVORITE_HASH)
+                collected_hashes.add(release["digests"][FAVORITE_HASH])
+            return self.prepend_hash_types(collected_hashes, FAVORITE_HASH)
         except (ValueError, KeyError, ConnectionError):
             if self.project.s.is_verbose():
-                click_echo(
+                click.echo(
                     "{}: Error generating hash for {}".format(
-                        crayons.red("Warning", bold=True), ireq.name
+                        click.style("Warning", bold=True, fg="red"), ireq.name
                     ),
                     err=True,
                 )
@@ -771,9 +802,7 @@ class Resolver:
         return self.hashes
 
     def _get_hash_from_link(self, link):
-        from pipenv.vendor.pip_shims import shims
-
-        if link.hash and link.hash_name == shims.FAVORITE_HASH:
+        if link.hash and link.hash_name == FAVORITE_HASH:
             return f"{link.hash_name}:{link.hash}"
 
         return self.hash_cache.get_hash(link)
@@ -849,15 +878,23 @@ def actually_resolve_deps(
     sources,
     clear,
     pre,
+    category,
     req_dir=None,
 ):
     if not req_dir:
         req_dir = create_tracked_tempdir(suffix="-requirements", prefix="pipenv-")
-    warning_list = []
 
     with warnings.catch_warnings(record=True) as warning_list:
         resolver = Resolver.create(
-            deps, project, index_lookup, markers_lookup, sources, req_dir, clear, pre
+            deps,
+            project,
+            index_lookup,
+            markers_lookup,
+            sources,
+            req_dir,
+            clear,
+            pre,
+            category,
         )
         resolver.resolve()
         hashes = resolver.resolve_hashes()
@@ -908,11 +945,11 @@ def venv_resolve_deps(
     deps,
     which,
     project,
+    category,
     pre=False,
     clear=False,
     allow_global=False,
     pypi_mirror=None,
-    dev=False,
     pipfile=None,
     lockfile=None,
     keep_outdated=False,
@@ -941,27 +978,21 @@ def venv_resolve_deps(
     :return: Nothing
     :rtype: None
     """
-
-    import json
-
     from pipenv import resolver
-    from pipenv._compat import decode_for_output
-    from pipenv.vendor.vistir.compat import NamedTemporaryFile, Path
 
-    results = []
-    pipfile_section = "dev-packages" if dev else "packages"
-    lockfile_section = "develop" if dev else "default"
+    lockfile_section = get_lockfile_section_using_pipfile_category(category)
+
     if not deps:
         if not project.pipfile_exists:
             return None
-        deps = project.parsed_pipfile.get(pipfile_section, {})
+        deps = project.parsed_pipfile.get(category, {})
     if not deps:
         return None
 
     if not pipfile:
-        pipfile = getattr(project, pipfile_section, {})
+        pipfile = getattr(project, category, {})
     if not lockfile:
-        lockfile = project._lockfile
+        lockfile = project._lockfile(categories=[category])
     req_dir = create_tracked_tempdir(prefix="pipenv", suffix="requirements")
     cmd = [
         which("python", allow_global=allow_global),
@@ -973,9 +1004,12 @@ def venv_resolve_deps(
         cmd.append("--clear")
     if allow_global:
         cmd.append("--system")
-    if dev:
-        cmd.append("--dev")
-    target_file = NamedTemporaryFile(prefix="resolver", suffix=".json", delete=False)
+    if category:
+        cmd.append("--category")
+        cmd.append(category)
+    target_file = tempfile.NamedTemporaryFile(
+        prefix="resolver", suffix=".json", delete=False
+    )
     target_file.close()
     cmd.extend(["--write", make_posix(target_file.name)])
     with temp_environ():
@@ -1000,28 +1034,32 @@ def venv_resolve_deps(
             # dependency resolution on them, so we are including this step inside the
             # spinner context manager for the UX improvement
             sp.write(decode_for_output("Building requirements..."))
-            deps = convert_deps_to_pip(deps, project, r=False, include_index=True)
+            deps = convert_deps_to_pip(deps, project, include_index=True)
             constraints = set(deps)
-            os.environ["PIPENV_PACKAGES"] = str("\n".join(constraints))
+            with tempfile.NamedTemporaryFile(
+                mode="w+", prefix="pipenv", suffix="constraints.txt", delete=False
+            ) as constraints_file:
+                constraints_file.write(str("\n".join(constraints)))
+            cmd.append("--constraints-file")
+            cmd.append(constraints_file.name)
             sp.write(decode_for_output("Resolving dependencies..."))
             c = resolve(cmd, sp, project=project)
-            results = c.stdout.strip()
             if c.returncode == 0:
                 sp.green.ok(environments.PIPENV_SPINNER_OK_TEXT.format("Success!"))
                 if not project.s.is_verbose() and c.stderr.strip():
-                    click_echo(crayons.yellow(f"Warning: {c.stderr.strip()}"), err=True)
+                    click.echo(click.style(f"Warning: {c.stderr.strip()}"), err=True)
             else:
                 sp.red.fail(
                     environments.PIPENV_SPINNER_FAIL_TEXT.format("Locking Failed!")
                 )
-                click_echo(f"Output: {c.stdout.strip()}", err=True)
-                click_echo(f"Error: {c.stderr.strip()}", err=True)
+                click.echo(f"Output: {c.stdout.strip()}", err=True)
+                click.echo(f"Error: {c.stderr.strip()}", err=True)
     try:
         with open(target_file.name) as fh:
             results = json.load(fh)
     except (IndexError, json.JSONDecodeError):
-        click_echo(c.stdout.strip(), err=True)
-        click_echo(c.stderr.strip(), err=True)
+        click.echo(c.stdout.strip(), err=True)
+        click.echo(c.stderr.strip(), err=True)
         if os.path.exists(target_file.name):
             os.unlink(target_file.name)
         raise RuntimeError("There was a problem with locking.")
@@ -1040,6 +1078,7 @@ def resolve_deps(
     python=False,
     clear=False,
     pre=False,
+    category=None,
     allow_global=False,
     req_dir=None,
 ):
@@ -1070,6 +1109,7 @@ def resolve_deps(
                 sources,
                 clear,
                 pre,
+                category,
                 req_dir=req_dir,
             )
         except RuntimeError:
@@ -1098,6 +1138,7 @@ def resolve_deps(
                     sources,
                     clear,
                     pre,
+                    category,
                     req_dir=req_dir,
                 )
             except RuntimeError:
@@ -1106,10 +1147,7 @@ def resolve_deps(
 
 
 @lru_cache()
-def get_pipenv_sitedir():
-    # type: () -> Optional[str]
-    import pkg_resources
-
+def get_pipenv_sitedir() -> Optional[str]:
     site_dir = next(
         iter(d for d in pkg_resources.working_set if d.key.lower() == "pipenv"), None
     )
